import { Component, Host, h, Prop, State, Element, Watch, writeTask } from '@stencil/core';
import { RemoteUpdateType, } from '../../types/dyte-client';
import DyteNotificationsAudio from '../../lib/notification';
import { formatName, shorten } from '../../utils/string';
import { defaultIconPack } from '../../lib/icons';
import { useLanguage } from '../../lib/lang';
import storeState from '../../lib/store';
/**
 * A component which handles notifications.
 *
 * You can configure which notifications you want to see and which ones you want to hear.
 * There are also certain limits which you can set as well.
 */
export class DyteNotifications {
  constructor() {
    this.notificationPermissions = {
      participant_joined: false,
      participant_left: false,
      chat: false,
      polls: true,
    };
    this.soundPermissions = {
      participant_joined: false,
      participant_left: false,
      chat: false,
      polls: true,
    };
    /** Language */
    this.t = useLanguage();
    /** Icon pack */
    this.iconPack = defaultIconPack;
    this.notifications = [];
    this.onRemoteUpdate = ({ payload, type }) => {
      if (type === RemoteUpdateType.INCOMING_REQUEST_ACCEPTED) {
        let remotePeer = this.meeting.participants.joined.get(payload.request.remotePeerId);
        this.add({
          id: `message-${Math.random().toString(36)}`,
          icon: this.iconPack.chat,
          message: `Granted remote control to ${remotePeer.name}`,
          duration: 2000,
        });
      }
      if (type === RemoteUpdateType.OUTGOING_REQUEST_ACCEPTED) {
        let hostPeer = this.meeting.participants.joined.get(payload.request.hostPeerId);
        this.add({
          id: `message-${Math.random().toString(36)}`,
          icon: this.iconPack.chat,
          message: `${hostPeer.name} has granted remote control.`,
          duration: 2000,
        });
      }
      if (type === RemoteUpdateType.REQUEST_RECEIVED) {
        let remotePeer = this.meeting.participants.joined.get(payload.request.remotePeerId);
        this.add({
          id: `message-${Math.random().toString(36)}`,
          icon: this.iconPack.chat,
          message: `${remotePeer.name} has requested for remote control.`,
          duration: 2000,
        });
      }
      if (type === RemoteUpdateType.INCOMING_REQUEST_ENDED ||
        type === RemoteUpdateType.OUTGOING_REQUEST_ENDED) {
        this.add({
          id: `message-${Math.random().toString(36)}`,
          icon: this.iconPack.chat,
          message: `Existing remote control has been terminated.`,
          duration: 2000,
        });
      }
      if (type === RemoteUpdateType.REQUEST_SENT) {
        let hostPeer = this.meeting.participants.joined.get(payload.request.hostPeerId);
        this.add({
          id: `message-${Math.random().toString(36)}`,
          icon: this.iconPack.chat,
          message: `Sent remote control request to ${hostPeer.name}`,
          duration: 2000,
        });
      }
    };
    this.onDyteNotification = (e) => {
      this.add(e.detail);
    };
  }
  connectedCallback() {
    if (typeof document !== 'undefined') {
      document === null || document === void 0 ? void 0 : document.addEventListener('dyteNotification', this.onDyteNotification);
    }
    this.meetingChanged(this.meeting);
    this.configChanged(this.config);
  }
  disconnectedCallback() {
    if (typeof document !== 'undefined') {
      document === null || document === void 0 ? void 0 : document.removeEventListener('dyteNotification', this.onDyteNotification);
    }
    if (this.meeting == null)
      return;
    this.participantJoinedListener &&
      this.meeting.participants.joined.removeListener('participantJoined', this.participantJoinedListener);
    this.participantLeftListener &&
      this.meeting.participants.joined.removeListener('participantLeft', this.participantLeftListener);
    this.chatUpdateListener &&
      this.meeting.chat.removeListener('chatUpdate', this.chatUpdateListener);
    this.pollUpdateListener &&
      this.meeting.polls.removeListener('pollsUpdate', this.pollUpdateListener);
    this.networkUpdateListener &&
      this.meeting.meta.removeListener('disconnected', this.networkUpdateListener);
    this.networkUpdateListener &&
      this.meeting.meta.removeListener('connected', this.networkUpdateListener);
    this.meeting.remote.removeListener('remoteUpdate', this.onRemoteUpdate);
  }
  meetingChanged(meeting) {
    if (meeting != null) {
      this.audio = new DyteNotificationsAudio();
      this.participantJoinedListener = (participant) => {
        if (this.notificationPermissions.participant_joined) {
          this.add({
            id: `${participant.id}-joined`,
            message: `${formatName(participant.name)} just joined`,
            image: participant.picture,
            duration: 2100,
          });
        }
        if (this.soundPermissions.participant_joined && this.canPlayParticipantJoinedSound()) {
          this.audio.play('joined');
        }
      };
      this.participantLeftListener = (participant) => {
        if (this.notificationPermissions.participant_left) {
          this.add({
            id: `${participant.id}-left`,
            message: `${formatName(participant.name)} left`,
            image: participant.picture,
            duration: 2100,
          });
        }
      };
      this.chatUpdateListener = ({ message }) => {
        if (message != null) {
          if (message.userId === meeting.self.userId) {
            return;
          }
          if (message.type === 'text') {
            if (this.notificationPermissions.chat) {
              this.add({
                id: `message-${Math.random().toString(36)}`,
                icon: this.iconPack.chat,
                message: `${message.displayName}: ${shorten(message.message, 20)}`,
                duration: 2000,
              });
            }
            if (this.soundPermissions.chat && this.canPlayChatSound()) {
              this.audio.play('message');
            }
          }
        }
      };
      this.pollUpdateListener = ({ polls }) => {
        if (this.notificationPermissions.polls &&
          this.meeting.self.userId !== polls[polls.length - 1].createdByUserId) {
          this.add({
            id: `poll-${Math.random().toString(36)}`,
            icon: this.iconPack.poll,
            message: `New poll created by ${polls[polls.length - 1].createdBy}`,
            duration: 2000,
          });
        }
        if (this.soundPermissions.polls &&
          this.meeting.self.userId !== polls[polls.length - 1].createdByUserId &&
          this.canPlayChatSound()) {
          this.audio.play('message');
        }
      };
      this.deviceUpdateListener = ({ device, preview }) => {
        if (preview)
          return;
        if (device.kind === 'audiooutput') {
          this.audio.setDevice(device.deviceId);
        }
      };
      this.networkUpdateListener = ({ event }) => {
        this.remove('network');
        let reconnectDuration;
        if (event === 'disconnected') {
          reconnectDuration = 20000;
          this.add({
            id: 'network',
            icon: this.iconPack.disconnected,
            message: this.t('Connection lost. Trying to reconnect...'),
            duration: reconnectDuration,
          });
          this.disconnectTimeout = setTimeout(() => {
            var _a;
            if (this.states != null) {
              this.states.roomLeftState = 'disconnected';
            }
            storeState.roomLeftState = 'disconnected';
            (_a = this.meeting) === null || _a === void 0 ? void 0 : _a.leaveRoom();
          }, reconnectDuration);
        }
        else if (event === 'connected') {
          reconnectDuration = 3000;
          this.add({
            id: `network`,
            icon: this.iconPack.wifi,
            message: this.t('Connection restored'),
            duration: reconnectDuration,
          });
          clearTimeout(this.disconnectTimeout);
        }
      };
      const currentDevices = meeting.self.getCurrentDevices();
      if (currentDevices.speaker != null) {
        this.audio.setDevice(currentDevices.speaker.deviceId);
      }
      meeting.participants.joined.addListener('participantJoined', this.participantJoinedListener);
      meeting.participants.joined.addListener('participantLeft', this.participantLeftListener);
      meeting.chat.addListener('chatUpdate', this.chatUpdateListener);
      meeting.polls.addListener('pollsUpdate', this.pollUpdateListener);
      meeting.self.addListener('deviceUpdate', this.deviceUpdateListener);
      meeting.meta.addListener('disconnected', () => this.networkUpdateListener({ event: 'disconnected' }));
      meeting.meta.addListener('connected', () => this.networkUpdateListener({ event: 'connected' }));
      meeting.remote.addListener('remoteUpdate', this.onRemoteUpdate);
    }
  }
  configChanged(config) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
    if (config != null) {
      this.notificationPermissions.participant_joined =
        (_b = (_a = config.config) === null || _a === void 0 ? void 0 : _a.notifications) === null || _b === void 0 ? void 0 : _b.includes('participant_joined');
      this.notificationPermissions.participant_left =
        (_d = (_c = config.config) === null || _c === void 0 ? void 0 : _c.notifications) === null || _d === void 0 ? void 0 : _d.includes('participant_left');
      this.notificationPermissions.chat = (_f = (_e = config.config) === null || _e === void 0 ? void 0 : _e.notifications) === null || _f === void 0 ? void 0 : _f.includes('chat');
      this.soundPermissions.participant_joined =
        (_h = (_g = config.config) === null || _g === void 0 ? void 0 : _g.notification_sounds) === null || _h === void 0 ? void 0 : _h.includes('participant_joined');
      this.soundPermissions.participant_left =
        (_k = (_j = config.config) === null || _j === void 0 ? void 0 : _j.notification_sounds) === null || _k === void 0 ? void 0 : _k.includes('participant_left');
      this.soundPermissions.chat = (_m = (_l = config.config) === null || _l === void 0 ? void 0 : _l.notification_sounds) === null || _m === void 0 ? void 0 : _m.includes('chat');
      this.participantChatMessageSoundNotificationLimit =
        (_o = config.config) === null || _o === void 0 ? void 0 : _o.participant_chat_message_sound_notification_limit;
      this.participantJoinedSoundNotificationLimit =
        (_p = config.config) === null || _p === void 0 ? void 0 : _p.participant_joined_sound_notification_limit;
    }
  }
  statesUpdate(states) {
    var _a, _b;
    if (states != null) {
      const notificationSoundsEnabled = !states.prefs.muteNotificationSounds;
      const configSoundPermission = (_b = (_a = this.config) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.notification_sounds;
      for (const permission in this.soundPermissions) {
        if (configSoundPermission === null || configSoundPermission === void 0 ? void 0 : configSoundPermission.includes(permission)) {
          this.soundPermissions[permission] = notificationSoundsEnabled;
        }
      }
    }
  }
  add(notification) {
    // show notifications only if tab is in focus and a maximum of 5 at a time
    if (document.visibilityState === 'visible' && this.notifications.length < 5) {
      // adds new notification to start of array so they appear at the bottom
      this.notifications = [...this.notifications, notification];
    }
  }
  remove(id) {
    this.notifications = this.notifications.filter((notification) => notification.id !== id);
  }
  handleDismiss(e) {
    e.stopPropagation();
    const id = e.detail;
    const el = this.host.shadowRoot.querySelector(`[data-id="${id}"]`);
    // exit animation
    el === null || el === void 0 ? void 0 : el.classList.add('exit');
    setTimeout(() => {
      writeTask(() => {
        this.remove(id);
      });
    }, 400);
  }
  canPlayParticipantJoinedSound() {
    return (this.participantJoinedSoundNotificationLimit == undefined ||
      this.participantJoinedSoundNotificationLimit <= 0 ||
      this.meeting.participants.count <= this.participantJoinedSoundNotificationLimit);
  }
  canPlayChatSound() {
    return (this.participantChatMessageSoundNotificationLimit == undefined ||
      this.participantChatMessageSoundNotificationLimit <= 0 ||
      this.meeting.participants.count <= this.participantChatMessageSoundNotificationLimit);
  }
  render() {
    return (h(Host, null, this.notifications.map((notification) => (h("dyte-notification", { size: this.size, key: notification.id, "data-id": notification.id, notification: notification, onDyteNotificationDismiss: (e) => this.handleDismiss(e) })))));
  }
  static get is() { return "dyte-notifications"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dyte-notifications.css"]
  }; }
  static get styleUrls() { return {
    "$": ["dyte-notifications.css"]
  }; }
  static get properties() { return {
    "meeting": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Meeting",
        "resolved": "DyteClient",
        "references": {
          "Meeting": {
            "location": "import",
            "path": "../../types/dyte-client"
          }
        }
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Meeting object"
      }
    },
    "states": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "States",
        "resolved": "States",
        "references": {
          "States": {
            "location": "import",
            "path": "../../types/props"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "States object"
      }
    },
    "config": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "UIConfig",
        "resolved": "UIConfig",
        "references": {
          "UIConfig": {
            "location": "import",
            "path": "../../types/ui-config"
          }
        }
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Config object"
      }
    },
    "t": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(key: \"audio\" | \"video\" | \"chat\" | \"participants\" | \"polls\" | \"plugins\" | \"mic_off\" | \"mic_on\" | \"video_off\" | \"video_on\" | \"left_meeting\" | \"share_screen_start\" | \"share_screen_stop\" | \"leave\" | \"dismiss\" | \"page\" | \"settings\" | \"connection\" | \"leave_confirmation\" | \"end_meeting_for_all\" | \"cancel\" | \"perm_browser_denied\" | \"perm_sys_denied\" | \"perm_denied_title\" | \"perm_denied_text\" | (string & {})) => string",
        "resolved": "(key: \"audio\" | \"video\" | \"chat\" | \"participants\" | \"polls\" | \"plugins\" | \"mic_off\" | \"mic_on\" | \"video_off\" | \"video_on\" | \"left_meeting\" | \"share_screen_start\" | \"share_screen_stop\" | \"leave\" | \"dismiss\" | \"page\" | \"settings\" | \"connection\" | \"leave_confirmation\" | \"end_meeting_for_all\" | \"cancel\" | \"perm_browser_denied\" | \"perm_sys_denied\" | \"perm_denied_title\" | \"perm_denied_text\" | (string & {})) => string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Language"
      },
      "defaultValue": "useLanguage()"
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Size",
        "resolved": "\"lg\" | \"md\" | \"sm\" | \"xl\"",
        "references": {
          "Size": {
            "location": "import",
            "path": "../../types/props"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Size"
      },
      "attribute": "size",
      "reflect": true
    },
    "iconPack": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "IconPack",
        "resolved": "{ people: string; chat: string; poll: string; participants: string; rocket: string; call_end: string; share: string; mic_on: string; mic_off: string; video_on: string; video_off: string; share_screen_start: string; share_screen_stop: string; share_screen_person: string; clock: string; dismiss: string; send: string; search: string; more_vertical: string; chevron_down: string; chevron_up: string; chevron_left: string; chevron_right: string; settings: string; wifi: string; speaker: string; download: string; full_screen_maximize: string; full_screen_minimize: string; copy: string; attach: string; image: string; emoji_multiple: string; vertical_scroll: string; vertical_scroll_disabled: string; image_off: string; disconnected: string; wand: string; recording: string; subtract: string; stop_recording: string; warning: string; pin: string; pin_off: string; spinner: string; }",
        "references": {
          "IconPack": {
            "location": "import",
            "path": "../../lib/icons"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Icon pack"
      },
      "defaultValue": "defaultIconPack"
    }
  }; }
  static get states() { return {
    "notifications": {}
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "meeting",
      "methodName": "meetingChanged"
    }, {
      "propName": "config",
      "methodName": "configChanged"
    }, {
      "propName": "states",
      "methodName": "statesUpdate"
    }]; }
}
