import spacingScale from '../theme/spacing-scale.json';
import { THEMES } from '../theme/presets/themes';
import { BORDER_WIDTHS, BORDER_RADII } from '../theme/presets/border';
import { hexToRGB } from './color';
const provideGoogleFont = (fontFamily) => {
  const weights = [400, 500, 700];
  const links = [];
  if (document.querySelector('link[data-dyte-font]') == null) {
    /**
     * Adds the following preconnect link tags for faster google font loading
     * <link rel="preconnect" href="https://fonts.googleapis.com">
     * <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
     */
    const p1 = document.createElement('link');
    p1.rel = 'preconnect';
    p1.setAttribute('data-dyte-font', 'true');
    p1.href = 'https://fonts.googleapis.com';
    const p2 = p1.cloneNode(true);
    p2.href = 'https://fonts.gstatic.com';
    p2.crossOrigin = '';
    links.push(p1, p2);
  }
  const link = document.createElement('link');
  link.rel = 'stylesheet';
  link.setAttribute('data-dyte-font', fontFamily);
  link.href = encodeURI(`https://fonts.googleapis.com/css2?family=${fontFamily}:wght@${weights.join(';')}&display=swap`);
  links.push(link);
  document.head.append(...links);
};
const provideSpacing = (el, spacingBase) => {
  spacingScale.forEach((space) => {
    el.style.setProperty(`--dyte-space-${space}`, `${space * spacingBase}px`);
  });
};
const provideBorderWidth = (el, borderWidth) => {
  if (borderWidth in BORDER_WIDTHS) {
    const widths = BORDER_WIDTHS[borderWidth];
    Object.keys(widths).forEach((type) => {
      el.style.setProperty(`--dyte-border-width-${type}`, `${widths[type]}px`);
    });
  }
};
const provideBorderRadius = (el, borderRadius) => {
  if (borderRadius in BORDER_RADII) {
    const radii = BORDER_RADII[borderRadius];
    Object.keys(radii).forEach((type) => {
      el.style.setProperty(`--dyte-border-radius-${type}`, `${radii[type]}px`);
    });
  }
};
const provideColors = (el, colors) => {
  Object.keys(colors).forEach((color) => {
    const colorValue = colors[color];
    if (typeof colorValue === 'string') {
      if (color === 'text' || color === 'text-on-brand') {
        const rgb = hexToRGB(colorValue).join(' ');
        el.style.setProperty(`--dyte-colors-${color}-1000`, `${rgb}`);
        el.style.setProperty(`--dyte-colors-${color}-900`, `${rgb} / 0.88`);
        el.style.setProperty(`--dyte-colors-${color}-800`, `${rgb} / 0.76`);
        el.style.setProperty(`--dyte-colors-${color}-700`, `${rgb} / 0.64`);
        el.style.setProperty(`--dyte-colors-${color}-600`, `${rgb} / 0.52`);
      }
      else {
        const rgb = hexToRGB(colorValue).join(' ');
        el.style.setProperty(`--dyte-colors-${color}`, rgb);
      }
    }
    else if (typeof colorValue === 'object') {
      Object.keys(colorValue).forEach((shade) => {
        const shadeValue = colorValue[shade];
        const rgb = hexToRGB(shadeValue).join(' ');
        el.style.setProperty(`--dyte-colors-${color}-${shade}`, rgb);
      });
    }
  });
};
/**
 * Provides the design system new tokens to consume values from for styling the Dyte UI Kit's UI.
 * @param el The element/node you want to _provide_ Dyte's design system.
 * @param tokens The design tokens you want to updated.
 */
export const provideDyteDesignSystem = (el, { spacingBase, borderRadius, borderWidth, colors, fontFamily, googleFont, theme }) => {
  if (typeof el !== 'object') {
    throw new Error('[dyte-ui-kit] (provideDyteDesignSystem): Passed element is not a valid HTML Element');
  }
  if (typeof googleFont === 'string') {
    provideGoogleFont(googleFont);
    el.style.setProperty('--dyte-font-family', googleFont);
  }
  if (typeof fontFamily === 'string') {
    el.style.setProperty('--dyte-font-family', fontFamily);
  }
  if (typeof spacingBase === 'number') {
    provideSpacing(el, spacingBase);
  }
  if (typeof borderWidth === 'string') {
    provideBorderWidth(el, borderWidth);
  }
  if (typeof borderRadius === 'string') {
    provideBorderRadius(el, borderRadius);
  }
  if (typeof theme === 'string') {
    const colors = THEMES[theme];
    if (colors != null)
      provideColors(el, colors);
  }
  if (typeof colors === 'object') {
    provideColors(el, colors);
  }
};
