import { r as registerInstance, h, g as getElement } from './index-74ddb327.js';
import { u as useLanguage, g as getLangData } from './index-a214afbc.js';
import { d as defaultIconPack } from './default-icon-pack-eabba256.js';
import { d as defaultConfig } from './default-ui-config-c5d610a7.js';
import { R as Render } from './index-f7bdca7a.js';
import { p as provideDyteDesignSystem, g as generateConfig } from './config-4902b022.js';
import { g as getUserPreferences } from './user-prefs-be1700cb.js';
import { s as state } from './store-79928517.js';
import { i as isV2Theme } from './versionTwoApis-e31e770d.js';
import './default-language-04851639.js';
import './graceful-storage-33bc316d.js';

const getIconPack = async (url) => {
  // check for both null/undefined
  if (url == null) {
    return defaultIconPack;
  }
  try {
    const res = await fetch(url);
    if (!res.ok) {
      return defaultIconPack;
    }
    // merge defaultIconPack with the received iconPack so as to
    // fill the missing icons with default ones
    return Object.assign({}, defaultIconPack, await res.json());
  }
  catch (_) {
    return defaultIconPack;
  }
};

const sm = 640;
const md = 768;
const lg = 1080;
const xl = 2160;
const breakpoints = {
	sm: sm,
	md: md,
	lg: lg,
	xl: xl
};

/**
 * Get the screen breakpoint from a given width
 * @param width The width of the container
 * @returns The screen breakpoint value
 */
const getSize = (width) => {
  if (width >= breakpoints.lg)
    return 'lg';
  else if (width >= breakpoints.md)
    return 'md';
  else
    return 'sm';
};

const dyteMeetingCss = ":host{box-sizing:border-box;display:flex;flex-direction:column;font-family:var(--dyte-font-family, sans-serif);--tw-bg-opacity:1;background-color:rgba(var(--dyte-colors-background-1000, 8 8 8) / var(--tw-bg-opacity));color:rgb(var(--dyte-colors-text-900, 255 255 255 / 0.88));overflow:hidden;position:fixed;top:var(--dyte-space-0, 0px);right:var(--dyte-space-0, 0px);bottom:var(--dyte-space-0, 0px);left:var(--dyte-space-0, 0px);height:100%;width:100%}:host([mode='fill']){position:relative}";

const DyteMeeting = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.roomJoinedListener = () => {
      this.setStates({ meeting: 'joined' });
      state.meeting = 'joined';
    };
    this.roomLeftListener = ({ state: state$1 }) => {
      const states = this.states || state;
      if ((states === null || states === void 0 ? void 0 : states.roomLeftState) === 'disconnected') {
        this.setStates({ meeting: 'ended' });
        state.meeting = 'ended';
        return;
      }
      this.setStates({ meeting: 'ended', roomLeftState: state$1 });
      state.meeting = 'ended';
      state.roomLeftState = state$1;
    };
    this.mediaPermissionErrorListener = ({ kind }) => {
      if (['audio', 'video'].includes(kind)) {
        this.setStates({ activePermissionsMessage: true });
        state.activePermissionsMessage = true;
      }
    };
    /** Whether to load config from preset */
    this.loadConfigFromPreset = true;
    /** Whether to apply the design system on the document root from config */
    this.applyDesignSystem = true;
    /** Fill type */
    this.mode = 'fixed';
    /** Language */
    this.language = 'en';
    /** UI Config */
    this.config = defaultConfig;
    this.states = {
      meeting: 'idle',
      prefs: {
        mirrorVideo: true,
        muteNotificationSounds: false,
        autoScroll: true,
      },
    };
    this.iconPack = defaultIconPack;
    this.t = useLanguage();
  }
  connectedCallback() {
    var _a;
    this.resizeObserver = new ResizeObserver(() => this.handleResize());
    this.resizeObserver.observe(this.host);
    if (this.applyDesignSystem &&
      ((_a = this.config) === null || _a === void 0 ? void 0 : _a.designTokens) != null &&
      typeof document !== 'undefined') {
      provideDyteDesignSystem(document.documentElement, this.config.designTokens);
    }
    this.meetingChanged(this.meeting);
    this.iconPackUrlChanged(this.iconPackUrl);
    this.languageChanged(this.language);
    this.initializePreferences();
  }
  disconnectedCallback() {
    var _a, _b;
    this.resizeObserver.disconnect();
    (_a = this.meeting) === null || _a === void 0 ? void 0 : _a.self.removeListener('roomJoined', this.roomJoinedListener);
    (_b = this.meeting) === null || _b === void 0 ? void 0 : _b.self.removeListener('roomLeft', this.roomLeftListener);
  }
  meetingChanged(meeting) {
    var _a;
    if (meeting != null) {
      if (this.loadConfigFromPreset && meeting.self.suggestedTheme != null) {
        const theme = meeting.self.suggestedTheme;
        const { config, data } = isV2Theme(theme)
          ? { config: defaultConfig, data: { showSetupScreen: true } }
          : generateConfig(theme);
        if (this.config === defaultConfig) {
          // only override the config if the object is same as defaultConfig
          // which means it's a different object passed via prop
          this.config = config;
        }
        if (this.showSetupScreen == null) {
          // only override this value if the prop isn't passed
          this.showSetupScreen = data.showSetupScreen;
        }
      }
      meeting.self.addListener('roomJoined', this.roomJoinedListener);
      meeting.self.addListener('roomLeft', this.roomLeftListener);
      meeting.self.addListener('mediaPermissionError', this.mediaPermissionErrorListener);
      if (this.applyDesignSystem &&
        ((_a = this.config) === null || _a === void 0 ? void 0 : _a.designTokens) != null &&
        typeof document !== 'undefined') {
        provideDyteDesignSystem(document.documentElement, this.config.designTokens);
      }
      if (this.showSetupScreen) {
        this.states = Object.assign(Object.assign({}, this.states), { meeting: 'setup' });
        state.meeting = 'setup';
      }
      else {
        // join directly to the meeting
        meeting.joinRoom();
      }
    }
  }
  async iconPackUrlChanged(url) {
    this.iconPack = await getIconPack(url);
  }
  async languageChanged(lang) {
    this.t = useLanguage(await getLangData(lang));
  }
  listenState(e) {
    e.stopPropagation();
    this.setStates(e.detail);
  }
  initializePreferences() {
    const prefs = getUserPreferences();
    this.setStates({ prefs });
    state.prefs = prefs;
  }
  handleResize() {
    this.size = getSize(this.host.clientWidth);
  }
  setStates(states) {
    this.states = Object.assign(Object.assign({}, this.states), states);
  }
  render() {
    const defaults = {
      meeting: this.meeting,
      size: this.size,
      states: this.states || state,
      config: this.config,
      iconPack: this.iconPack,
      t: this.t,
    };
    return h(Render, { element: "dyte-meeting", defaults: defaults, asHost: true });
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "meeting": ["meetingChanged"],
    "iconPackUrl": ["iconPackUrlChanged"],
    "language": ["languageChanged"]
  }; }
};
DyteMeeting.style = dyteMeetingCss;

export { DyteMeeting as dyte_meeting };
